# coding=utf-8
'''
This parser converts message body to our structured list.
It reveals words, lines and paragraphs.
@author: Peeter Jyrviste
'''
import cStringIO, re
from BeautifulSoup import BeautifulSoup

# remove specified element comp from the list
def removeValuesFromList(list, comp):
    return [value for value in list if value != comp]

# replace special characters like &#60; or &lt; with corresponding symbols
def replaceSpecialChars(matchobj):   
    # this list can be easily extended
    if matchobj.group() == '&#34;' or matchobj.group() == '&quot;': 
        return '"'
    elif matchobj.group() == '&#39;' or matchobj.group() == '&apos;': 
        return "\\'"
    elif matchobj.group() == '&#38;' or matchobj.group() == '&amp;': 
        return '&'
    elif matchobj.group() == '&#60;' or matchobj.group() == '&lt;': 
        return '<'
    elif matchobj.group() == '&#62;' or matchobj.group() == '&gt;': 
        return '>'
    
    elif matchobj.group() == '&#245;' or matchobj.group() == '&otilde;': 
        return 'õ'
    elif matchobj.group() == '&#228;' or matchobj.group() == '&auml;': 
        return 'ä'
    elif matchobj.group() == '&#246;' or matchobj.group() == '&ouml;': 
        return 'ö'
    elif matchobj.group() == '&#252;' or matchobj.group() == '&uuml;': 
        return 'ü'
    elif matchobj.group() == '&#227;' or matchobj.group() == '&atilde;': 
        return 'ã'
    elif matchobj.group() == '&#229;' or matchobj.group() == '&aring;': 
        return 'å'
    elif matchobj.group() == '&#248;' or matchobj.group() == '&oslash;': 
        return 'ø'
    
    elif matchobj.group() == '&#213;' or matchobj.group() == '&Otilde;': 
        return 'Õ'
    elif matchobj.group() == '&#196;' or matchobj.group() == '&Auml;': 
        return 'Ä'
    elif matchobj.group() == '&#214;' or matchobj.group() == '&Ouml;': 
        return 'Ö'
    elif matchobj.group() == '&#220;' or matchobj.group() == '&Uuml;': 
        return 'Ü'
    elif matchobj.group() == '&#195;' or matchobj.group() == '&Atilde;': 
        return 'Ã'
    elif matchobj.group() == '&#197;' or matchobj.group() == '&Aring;': 
        return 'Å'
    elif matchobj.group() == '&#216;' or matchobj.group() == '&Oslash;': 
        return 'Ø'
    else:
        return ''

class BodyParser:
        
    # introduce new paragraphs after symbols .!?"
    def additionalParaSeparation(self, list):
        oldPara = [item for sublist in list for item in sublist]
        newPara = []
        result = []
        for line in oldPara:
            newPara.append(line)
            last = line.pop()
            if (last.endswith('.') or last.endswith('!') or \
                last.endswith('?') or last.endswith('"')):
                newPara = filter(lambda x:len(x) > 0, newPara)
                result.append(newPara)
                newPara = []
        return result

    # process Beautiful Soup output and return valid body structure
    def processList(self, list):
        temp = [] # here I store temporary lists for processing
        temp2 = []
        output = [] # the output double list
        # conversion from unicode to string
        for i in list:
            temp.append(str(i))
        for i in temp:
            # remove invalid Beautiful Soup temp2
            correct = re.sub(r'=\r\n', '', i)
            # replace =A0 characters from text with spaces, they are &nbsp;
            spaces = re.sub(r'=A0', ' ', correct)
            # replace =3D with the equal sign
            equal = re.sub(r'=3D', '=', spaces)
            # replace special characters like õ, ä, ö, ü, etc.
            special = re.sub('(&#?[a-zA-Z0-9]+;)', replaceSpecialChars, equal)
            temp2.append(special)
        # split by new line characters
        temp = []
        for i in temp2:
            temp.append(re.split(r'\n|\r', i))
        temp2 = []
        for i in temp:
            for word in i:
                temp2.append(word.split())
        # throw out empty elements and '<' and '>' symbols
        temp = []
        for i in temp2:
            temp.append(removeValuesFromList(i, '>'))
        temp2 = []
        for i in temp:
            temp2.append(removeValuesFromList(i, '<'))
        temp = []
        for i in temp2:
            if i:
                temp.append(i)
                output.append(temp)
                temp = []
        return output

    # extracts paragraph information
    def processParas(self, message):
        result = []
        file = cStringIO.StringIO()
        para = []
        paraCounter = 0
        for part in message.walk():
            if not part.is_multipart():
                if part.get_content_type() == 'text/plain':
                    file.write(part.get_payload())
                    file.seek(0) # go to the beginning
                    for line in file.readlines():
                        # replace =A0 characters from text with spaces, they are &nbsp;
                        spaces = re.sub(r'=A0', ' ', line)
                        # replace =3D with the equal sign
                        equal = re.sub(r'=3D', '=', spaces)
                        # get paragraphs by finding empty lines
                        splitLine = equal.split()
                        a = removeValuesFromList(splitLine, '>')
                        b = removeValuesFromList(a, '<')
                        if b:
                            para.append(b)
                        else:
                            if para:
                                result.append(para)
                                paraCounter = paraCounter + 1
                                para = []
                    if paraCounter <= 3:
                        result = BodyParser.additionalParaSeparation(self, result)
                if part.get_content_type() == 'text/html':
                    soup = BeautifulSoup.BeautifulSoup(''.join(part.get_payload()))
                    initialList = eval(str(soup.findAll(text=True)))
                    result = BodyParser.processList(self, initialList)
        file.close()
        return result
