'''
Email Graph class for storing the internal structure of messages as a graph
@author: Peeter Jyrviste
'''
import networkx as nx, mailbox, uuid, pickle
import BodyParser

# you must use the b flag, binary mode
def load(name):
    f=open(name,"rb")
    g = pickle.load(f)
    f.close()
    return g
    
# you must use the b flag, binary mode
def save(graph, name):
    f=open(name,"wb")
    pickle.dump(graph, f)
    f.close()

class EmailGraph:
    
    def processAttachments(self, message, id):
        for part in message.walk():
            filename = part.get_filename()
            if not filename:
                filename = part.get_param('name', None, 'content-type')
            if filename != None:
                self.G.add_node(str(filename))
                self.G.add_edge(id, str(filename), type='Attachment')
    
    def __init__(self, mailbox):
        self.mailbox = mailbox
        self.G = nx.MultiDiGraph()
        self.root = 0
        self.deriveGraph()
    
    def separateByCommas(self, string):
        tokens = string.split(',')
        for index, part in enumerate(tokens):
            part = part.strip()
            tokens[index] = part
        return tokens

    def processHeaderField(self, message, id, header, separationEnabled):
        if (header != 'Received'):
            if message.has_key(header):
                if(separationEnabled):
                    myList = EmailGraph.separateByCommas(self, message[header])
                    for i in myList:
                        self.G.add_node(i)
                        self.G.add_edge(id, i, type=header)
                else:
                    self.G.add_node(message[header])
                    self.G.add_edge(id, message[header], type=header)
        else:
            if message.has_key('Received'):
                received_lines = message.get_all('Received')
                for line in received_lines:
                    self.G.add_node(line)
                    self.G.add_edge(id, line, type=header)
    
    def displayGraph(self):
        print 'Nodes:'
        for node in self.G:
            print node
        print 'Edges:'
        print self.G.edges()
    
    def deriveGraph(self):
        myMbox = mailbox.mbox(self.mailbox)
        bodyParser = BodyParser.BodyParser()
        "Mailbox id"
        self.root = str(uuid.uuid4())
        self.G.add_node(self.root)
        for message in myMbox:
            "Unique identifier of the message, type string"
            id = str(uuid.uuid4())
            self.G.add_node(id)
            self.G.add_edge(self.root, id, type="Message")
            "Fill in information from header fields"
            EmailGraph.processHeaderField(self, message, id, 'Message-ID', True)
            EmailGraph.processHeaderField(self, message, id, 'From', True)
            EmailGraph.processHeaderField(self, message, id, 'To', True)
            EmailGraph.processHeaderField(self, message, id, 'Subject', False)
            EmailGraph.processHeaderField(self, message, id, 'Date', False)
            EmailGraph.processHeaderField(self, message, id, 'In-Reply-To', True)
            EmailGraph.processHeaderField(self, message, id, 'References', True)
            EmailGraph.processHeaderField(self, message, id, 'Comments', False)
            EmailGraph.processHeaderField(self, message, id, 'Keywords', True)
            EmailGraph.processHeaderField(self, message, id, 'Sender', True)
            EmailGraph.processHeaderField(self, message, id, 'Reply-To', True)
            EmailGraph.processHeaderField(self, message, id, 'Cc', True)
            EmailGraph.processHeaderField(self, message, id, 'Bcc', True)
            EmailGraph.processHeaderField(self, message, id, 'Received', None)
            EmailGraph.processAttachments(self, message, id)
            "This is the result of the BodyParser processing"
            "Type string but looks like a list"
            bodyString = str(bodyParser.processParas(message))
            self.G.add_node(bodyString)
            self.G.add_edge(id, bodyString, type='Body')
        return self.G
    
    "Returns root index, type string"
    def getRoot(self):
        return self.root
    
    "Returns the NXGraph, type MultiDiGraph"
    def getNXGraph(self):
        return self.G
    
    "Returns input parameter index, useless here"
    def getNodeData(self, index):
        return index
    
    "Returns the type(s) of a given edge tuple, type list"
    def getEdgeType(self, tuple):
        retArray = []
        list = self.G[tuple[0]][tuple[1]].values()
        for innerDict in list:
            retArray.append(innerDict.get('type'))
        return retArray
    
    "Returns all edges of a given type, type list"
    def getEdgesByType(self, edgeType):
        retArray = []
        for edge in self.G.edges():
            list = self.getEdgeType(edge)
            for i in list:
                if i == edgeType:
                    retArray.append(edge)
        return retArray
